from flask import Flask, redirect, url_for, render_template, request,session
from datetime import timedelta
from flask.helpers import flash
from flask_sqlalchemy import SQLAlchemy
from flask_bcrypt import Bcrypt

Usermanager = Flask(__name__)
Usermanager.config["SECRET_KEY"] = "hoanghd-secret-key"
Usermanager.config["SQLALCHEMY_DATABASE_URI"] = "mysql://hoanghd:Hoanghd164@192.168.13.200/usermanager"
Usermanager.permanent_session_lifetime = timedelta(minutes=1)

db_conn = SQLAlchemy(Usermanager)
bcrypt = Bcrypt(Usermanager)

class usermanager(db_conn.Model):
    id = db_conn.Column(db_conn.Integer, primary_key=True)
    username = db_conn.Column(db_conn.String(255))
    password = db_conn.Column(db_conn.String(255))
    email = db_conn.Column(db_conn.String(255))

    def __init__(self, username, password, email):
        self.username = username
        self.password = bcrypt.generate_password_hash(password).decode('utf-8')
        self.email = email

@Usermanager.route('/', methods=["POST", "GET"])
@Usermanager.route('/home', methods=["POST", "GET"])
def home_page():
    if "session_username" not in session:
        return redirect(url_for("username_login")) 
    
    if request.method == "POST":
        return redirect(url_for("username_logout"))
    
    return render_template("home.html")

@Usermanager.route('/login', methods=["POST", "GET"])
def username_login():
    with Usermanager.app_context():
        db_conn.create_all()

    if request.method == "POST":
        session.permanent = True
        form_username = request.form["username"]
        form_password = request.form["password"]
        if form_username:
            found_user = usermanager.query.filter_by(username=form_username).first()
            session["session_username"] = form_username
            if found_user:
                is_password = bcrypt.check_password_hash(found_user.password, form_password)
            if found_user and is_password:
                username_in_db = found_user.username
                session["session_username"] = username_in_db
                if is_password:
                    flash("Username login successfully")
                    return redirect(url_for("home_page"))
                else:
                    flash("Password is incorrect")
                    return redirect(url_for("username_login"))
            else:
                flash("Username %s does not exist!" %(session["session_username"]))
                return redirect(url_for("username_login"))
        else:
            if "session_username" in session:
                return redirect(url_for("home_page"))
            else:
                return redirect(url_for("username_login"))
        
    return render_template("login.html")

@Usermanager.route('/register', methods=["POST", "GET"])
def username_register():
    with Usermanager.app_context():
        db_conn.create_all()

    if request.method == "POST":
        session.permanent = True
        form_username = request.form["username"]
        form_password = request.form["password"]
        form_confirm_password = request.form["confirm_password"]
        form_email = request.form["email"]
        session.clear()

        if form_username:
            found_user = usermanager.query.filter_by(username=form_username).first()
            found_email = usermanager.query.filter_by(email=form_email).first()

            if found_user:
                username_in_db = found_user.username
                if form_username == username_in_db:
                    flash("User already exists!!!")
                    return redirect(url_for("username_register"))
            else:
                if found_email:
                    flash("Email already exists!!!")
                    return redirect(url_for("username_register"))
                else:
                    if form_password == form_confirm_password:
                        user_info = usermanager(form_username, form_password, form_email)
                        db_conn.session.add(user_info)
                        db_conn.session.commit()
                        flash("Username created successfully!!!")
                        return redirect(url_for("username_login"))
                    else:
                        flash("Password must be the same!")
                        return redirect(url_for("username_register"))

        if "session_username" in session:
            return redirect(url_for("home_page"))
        
    return render_template("register.html")

@Usermanager.route('/forgot_password', methods=["POST", "GET"])
def forgot_password():
    with Usermanager.app_context():
        db_conn.create_all()

    if request.method == "POST":
        session.permanent = True
        form_username = request.form["username"]
        form_new_password = request.form["new_password"]
        form_confirm_new_password = request.form["confirm_new_password"]
        form_email = request.form["email"]
        session.clear()
        
        if form_username:
            found_user = usermanager.query.filter_by(username=form_username).first()
            found_email = usermanager.query.filter_by(email=form_email).first()
            if found_user and found_email:
                if form_new_password == form_confirm_new_password:
                    found_user = usermanager.query.filter_by(username=form_username).first()
                    found_user.password = bcrypt.generate_password_hash(form_new_password).decode('utf-8')
                    db_conn.session.commit()
                    flash("Change password successfully!!!")
                    return redirect(url_for("username_login"))
                else:
                    flash("Password must be the same!!!")
                    return redirect(url_for("forgot_password"))
            else:
                if found_user:
                    email_in_db = found_user.email
                    if form_email != email_in_db:
                        flash("Username and email do not match on the system!!")
                        return redirect(url_for("forgot_password"))
                else:
                    flash("Username %s do not exist on the system!!" %(form_username))
                    return redirect(url_for("forgot_password"))
                
    return render_template("forgot_password.html")

@Usermanager.route('/logout')
def username_logout():
    session.clear()
    return redirect(url_for("username_login"))

if __name__ == "__main__":
    Usermanager.run(debug=True, host='0.0.0.0', port=5000)